/**
 * @file Fonctions qui implémentent les requetes relatives aux utilisateurs
 * @author ofacklam
 * @memberof GraphQL
 */

import { User } from "../object_resolvers/users";
import { User as UT, userData } from "../../ldap/export/user";
import { Group as GT, groupData } from "../../ldap/export/group";

import knex from "../../../db/knex_router";
import { searchTOLArgs, editProfileArgs } from "../typeDefs/queries";
import { ApolloError } from "apollo-server-core";
import {Cache} from "../../utils/cache";
import { GroupCollection, Tools, GroupSet } from "../../utils/tools";

export class UserModel {

    /**
     * @memberof GraphQL
     * @class UserModel
     * @summary Requetes relatives aux utilisateurs.
     * @classdesc Cette classe contient les méthodes implémentant les requetes relatives aux utilisateurs.
     * @arg {string} contextUser - L'identifiant de l'utilisateur du 'context'
     */
    constructor(contextUser: string) {
        this.contextUser = contextUser;
        this.cache = new Cache<User, string>();
    }

    protected contextUser: string;

    protected cache: Cache<User, string>;

    /**
     * @memberof GraphQL.UserModel#
     * @function getUser
     * @summary Fonction qui renvoie l'utilisateur demandé.
     * @arg {string} uid - Identifiant demandé.
     * @arg {boolean} load - Si load vaut false, les données ne seront pas chargées.
     * @return {Promise(User)} Renvoie l'utilisateur dont l'identifiant est 'uid'
     * @async
     * @rights connectedOrOnplatal
     */
    async getUser(uid: string, load: boolean = true): Promise<User> {
        return this.cache.get(uid, (id) => User.tryCreate(id, load));
    }

    /**
     * @memberof GraphQL.UserModel#
     * @function getMemberOf
     * @summary Fonction qui renvoie les groupes stricts du user.
     * @arg {string} uid - Identifiant demandé.
     * @return {Promise(GroupCollection)} Renvoie les identifiants des groupes dont ce user est membre strict.
     * @async
     * @rights connectedOrOnplatal
     */
    async getMemberOf(uid: string): Promise<GroupCollection> {
        let gd = new groupData();
        gd.members = [uid];
        let data = await GT.search(gd);

        let simple = new GroupSet(data);
        let meta = await Tools.metaGroupsOfGroups(simple);

        return {simpleGroups: simple, metaGroups: meta};
    }

    /**
     * @memberof GraphQL.UserModel#
     * @function getAdminOf
     * @summary Fonction qui renvoie les groupes dont le user est admin strict.
     * @arg {string} uid - Identifiant demandé.
     * @return {Promise(GroupCollection)} Renvoie les identifiants des groupes dont ce user est admin strict.
     * @async
     * @rights connectedOrOnplatal
     */
    async getAdminOf(uid: string): Promise<GroupCollection> {
        let gd = new groupData();
        gd.admins = [uid];
        let data = await GT.search(gd);

        let simple = new GroupSet(data);
        let meta = await Tools.metaGroupsOfGroups(simple);

        return { simpleGroups: simple, metaGroups: meta };
    }

    /**
     * @memberof GraphQL.UserModel#
     * @function getDislikes
     * @summary Fonction qui renvoie les groupes que le contextUser veut cacher.
     * @return {Promise(GroupSet)} Renvoie les identifiants des groupes que le contextUser veut cacher.
     * @async
     * @rights connected
     */
    async getDislikes(): Promise<GroupSet> {
        let data = await knex.select('gid').from('user_dislike_group').where('uid', this.contextUser);
        return new GroupSet(data.map(obj => obj.gid));
    }

    /**
     * @memberof GraphQL.UserModel#
     * @function searchTOL
     * @summary Fonction qui recherche dans le TOL
     * @arg {searchTOLArgs} args - les données de recherche
     * @return {Promise(User[])} Renvoie une liste d'utilisateurs
     * @async
     * @rights connectedOrOnplatal
     */
    async searchTOL(args: searchTOLArgs): Promise<User[]> {
        //TODO : correctly handle groups (in LDAP, a member can be stored as member, speaker or admin...)
        //SOLVED : LDAP stores the statuses correctly : admin > speaker > member

        const searchData = new userData();
        searchData.givenName = args.givenName;
        searchData.lastName = args.lastName;
        searchData.nickname = args.nickname;
        searchData.nationality = args.nationality;
        searchData.members = args.groups;
        searchData.phone = args.phone;
        searchData.mail = args.mail;
        searchData.address = args.address;

        const userList = await UT.search(searchData);
        return Promise.all(userList.map((uid) => this.getUser(uid, false)));
    }

    /**
     * @memberof GraphQL.UserModel#
     * @function editProfile
     * @summary Fonction qui modifie le profil et renvoie l'utilisateur
     * @arg {editProfileArgs} args - les données a modifier 
     * @return {Promise(User)} Renvoie l'utilisateur mis a jour
     * @async
     * @rights authenticated
     */
    async editProfile(args: editProfileArgs): Promise<User> {
        let data = await UT.peek(this.contextUser);

        //Modify some fields, keep the others
        let editArgs = new userData();
        editArgs.uid = data.uid;
        editArgs.password = data.password;
        editArgs.givenName = data.givenName;
        editArgs.lastName = data.lastName;
        editArgs.nickname = args.nickname; // <- this field is modified by user
        editArgs.gender = data.gender;
        editArgs.photo = data.photo;
        editArgs.phone = args.phone; // <- this field is modified
        editArgs.address = data.address; // WTF why can't this be changed ????
        editArgs.mail = args.mail; // <- this field is modified
        editArgs.birthdate = data.birthdate;
        editArgs.nationality = data.nationality;
        editArgs.admins = data.admins;
        editArgs.speakers = data.speakers;
        editArgs.members = data.members;
        editArgs.followers = data.followers;

        if(await UT.edit(editArgs)) {
            return this.getUser(data.uid, false);
        }
        else {
            throw new ApolloError("Edit failed");
        }
    }

}
